#!/bin/sh
#------------------------------------------------------------------------------
# /usr/local/bin/dyndns-update.sh - Update one DynDNS-provider
# part of opt_dyndns 2.1.17a, see documentation for licence
#
#  Copyright 2003-2004 Tobias Gruetzmacher
#
# Creation:    16.09.2003 tobig
# Last Update: $Id: dyndns-update.sh,v 1.6.2.2 2004/08/10 19:13:19 tobias Exp $
#------------------------------------------------------------------------------

if [ "$#" -ne "2" ]
then
  echo "Usage: $0 <hostname> <ip>"
  exit 1
fi

host="$1"
ip="$2"

# read which config file
conf="/etc/dyndns/dyndns.conf"
# which OPT_DynDNS version is this?
dyn_ver="2.1.17a"
# logdir
dyn_logdir="/var/log/dyndns"
# timeout for network-operations
dyn_timeout="60"

set -- `grep "^$host " $conf`

if [ "$#" -ne 5 ]
then
  echo "Error finding host '$host' in config file '$conf', aborting!"
  exit 1
fi

host="$1"
provider="$2"
user="$4"
pass="$5"

# divide hostname in subdomain and domain
oldIFS="$IFS"
IFS="."
set -- $host
subdom="$1"
shift
while [ "$#" -gt 2 ]
do
  subdom="$subdom.$1"
  shift
done
domain="$1.$2"
IFS="$oldIFS"

# read config for provider
. /etc/dyndns/provider.$provider

# update the dynamic hostname
echo "Setting up dynamic hostname on $host"

# logging? Use file descriptor 5, this is safe
if [ -d "$dyn_logdir" ]
then
  logfile="$dyn_logdir/$host"
  exec 5> "$logfile"
  echo "$host" 1>&5
  date 1>&5
else
  exec 5>&1
  echo "----- start answer -----"
fi

case "$provider_update_type" in
  http) # This is a plain http-request with or without basic auth
    {
    echo "GET $provider_url HTTP/1.1"
    echo "Host: $provider_host"
    echo "Connection: close"
    if [ "$provider_auth" != '' ]
    then
      echo "Authorization: Basic `echo -n "$provider_auth" | /usr/local/bin/base64`"
    fi
    echo "User-Agent: opt_dyndns/$dyn_ver fli4l@null.bogus"
    echo ""
    } | /usr/local/bin/netcat -w "$dyn_timeout" "$provider_host" "$provider_port" 1>&5
  ;;
  gnudip) # Use GnuDip over HTTP (Ask me if you need the direct socket connection)
    tmpfile="/tmp/dyndns.$$.gnudip"
    {
    echo "GET $provider_script HTTP/1.1"
    echo "Host: $provider_host"
    echo "Connection: close"
    echo "User-Agent: opt_dyndns/$dyn_ver fli4l@null.bogus"
    echo ""
    } | /usr/local/bin/netcat -w "$dyn_timeout" "$provider_host" "$provider_port" > $tmpfile
    # Sets meta_salt, meta_time and meta_sign
    eval "`sed -n 's/^<meta name="\([A-Za-z]*\)" content="\([A-Za-z0-9]*\)">.*$/meta_\1='"'\2'"'/; /^meta_/ p' $tmpfile`"
    set -- `echo -n "$pass" | md5`
    md5pass="$1"
    set -- `echo -n "$md5pass"."$meta_salt" | md5`
    endpass="$1"

    {
    echo "GET $provider_script?salt=$meta_salt&time=$meta_time&sign=$meta_sign&user=$user&pass=$endpass&domn=$domain&reqc=2 HTTP/1.1"
    echo "Host: $provider_host"
    echo "Connection: close"
    echo "User-Agent: opt_dyndns/$dyn_ver fli4l@null.bogus"
    echo ""
    } | /usr/local/bin/netcat -w "$dyn_timeout" "$provider_host" "$provider_port" 1>&5
    rm -f $tmpfile
  ;;
  netcat)
    echo -e "$provider_data" |
      /usr/local/bin/netcat -w "$dyn_timeout" "$provider_host" "$provider_port" 1>&5
  ;;
  *)
    echo "Error using provider $provider" 1>&5
esac

echo "------ end answer ------" 1>&5

[ "$logfile" ] && chmod a+r "$logfile"

# vi: set tabstop=2 shiftwidth=2 expandtab:
